import React, { useRef, useCallback, useState } from 'react';
import { connect, history } from 'umi';
import { PageContainer } from '@ant-design/pro-layout';
import ProTable from '@ant-design/pro-table';
import { PlusOutlined, MenuOutlined } from '@ant-design/icons';
import { Button, Popconfirm, Image, Space, notification } from 'antd';
import { sortableContainer, sortableElement, sortableHandle } from 'react-sortable-hoc';
import request, { queryTable } from '@/utils/request';
import arrayMove from 'array-move';

const DragHandle = sortableHandle(() => <MenuOutlined style={{ cursor: 'grab', color: '#999' }} />);

const SortableItem = sortableElement((props) => <tr {...props} />);
const SortableContainer = sortableContainer((props) => <tbody {...props} />);

const typeList = { post: '科普' };
// const stateList = { post: '科普' };

const BannerList = () => {
  const [, setLoading] = useState(false);
  const [dataSource, setDataSource] = useState([]);
  const ref = useRef();
  const handleBannerClick = useCallback((id) => {
    history.push(id ? `/cms/banner/edit?id=${id}` : '/cms/banner/edit');
  }, []);

  const actions = [
    <Button key="button" icon={<PlusOutlined />} type="primary" onClick={() => handleBannerClick()}>
      新建
    </Button>,
  ];

  const deleteBannerClick = (id) => {
    setLoading(true);

    request(`/banner/${id}`, { method: 'delete' })
      .then(() => {
        notification.success({ message: '删除成功' });
        ref.current.submit();
      })
      .catch((e) => {
        setLoading(false);
        notification.error({ message: e.message });
      });
  };

  const changeStatus = (row) => {
    setLoading(true);

    request(`/banner/${row.serialNo}`, {
      method: 'put',
      data: { ...row, status: row.status === 1 ? 0 : 1 },
    })
      .then(() => {
        notification.info({ message: '修改成功' });
        setLoading(false);
        ref.current.submit();
      })
      .catch((e) => {
        notification.error({ message: e.message });
        setLoading(false);
        return Promise.reject(e.message);
      });
  };

  const columns = [
    {
      title: '排序',
      dataIndex: 'sort',
      width: 60,
      className: 'drag-visible',
      hideInSearch: true,
      align: 'center',
      render: () => <DragHandle />,
    },
    {
      title: '主图',
      dataIndex: 'image',
      hideInSearch: true,

      align: 'center',
      render: (text) => <Image width={64} height={64} src={text} />,
    },
    {
      title: '名称',
      dataIndex: 'title',
      align: 'center',
      className: 'drag-visible',
    },
    {
      title: '板块',
      dataIndex: 'targetType',
      hideInSearch: true,
      align: 'center',
      render: (text) => typeList[text] || '',
    },

    {
      title: '状态',
      dataIndex: 'status',
      align: 'center',
      hideInSearch: true,

      valueEnum: {
        0: { text: '未启用', status: 'Default' },
        1: { text: '启用', status: 'Success' },
      },
    },
    {
      title: '操作',
      dataIndex: 'action',
      align: 'center',
      hideInSearch: true,
      render: (text, record) => (
        <Space size="middle">
          <a key="status" onClick={() => changeStatus(record)}>
            {record.status === 1 ? '弃用' : '启用'}
          </a>
          <a onClick={() => handleBannerClick(record.serialNo)}>编辑</a>
          <Popconfirm
            key="popconfirm"
            title={`确认删除吗?`}
            onConfirm={() => deleteBannerClick(record.serialNo)}
            okText="是"
            cancelText="否"
          >
            <a>删除</a>
          </Popconfirm>
        </Space>
      ),
    },
  ];

  const onSortEnd = ({ oldIndex, newIndex }) => {
    // const { dataSource } = this.state;
    console.log('Sorted items: ', dataSource, oldIndex, newIndex);
    if (oldIndex !== newIndex) {
      const newData = arrayMove([].concat(dataSource), oldIndex, newIndex).filter((el) => !!el);
      console.log('Sorted items: ', newData);
      //   this.setState({ dataSource: newData });
      setDataSource(newData);
    }
  };

  const DraggableContainer = (props) => (
    <SortableContainer
      useDragHandle
      disableAutoscroll
      helperClass="row-dragging"
      onSortEnd={onSortEnd}
      {...props}
    />
  );

  const DraggableBodyRow = ({ className, style, ...restProps }) => {
    // const { dataSource } = this.state;
    // function findIndex base on Table rowKey props and should always be a right array index
    console.log(restProps, dataSource, 'restProps');
    const index = dataSource?.findIndex((x) => x.serialNo === restProps['data-row-key']);
    return <SortableItem index={index} {...restProps} />;
  };

  const getData = (params, sort, filter) => {
    return queryTable('/banner')(params, sort, filter).then((res) => {
      setDataSource(res.data);
      return res;
    });
  };

  return (
    <PageContainer>
      <ProTable
        columns={columns}
        dataSource={dataSource}
        request={getData}
        postData={(data) => data}
        formRef={ref}
        rowKey="serialNo"
        headerTitle="科普列表"
        search={{
          labelWidth: '4em',
        }}
        components={{
          body: {
            wrapper: DraggableContainer,
            row: DraggableBodyRow,
          },
        }}
        toolBarRender={() => actions}
      />
    </PageContainer>
  );
};

export default connect((s) => ({
  typeList: s.post.typeList,
}))(BannerList);
